@@ -37,6 +37,7 @@ class ResponseMixin:
         self.__seqnum = 0
         self.__headers = {}
         self.set_header('Pragma', 'no-cache')
+        self.set_header('Cache-Control', 'no-cache')
         self.set_header('Content-Type', 'text/html')
 
     def get_header(self, name):
@@ -294,16 +295,19 @@ class Application(ResourceMixin):
         import traceback
         import linecache
         etype, value, tb = sys.exc_info()
-        list = traceback.format_exception(etype, value, tb)
-        pyexc = ''.join(list)
-        list = []
-        list.append('Template traceback (most recent call last):')
-        for tup in self.template_traceback(tb):
-            list.append('  File "%s", line %s, in %s' % tup)
-            line = linecache.getline(tup[0], tup[1]).rstrip()
-            if line:
-                list.append(line)
-        htmlexc = '\n'.join(list)
+        try:
+            list = traceback.format_exception(etype, value, tb)
+            pyexc = ''.join(list)
+            list = []
+            list.append('Template traceback (most recent call last):')
+            for tup in self.template_traceback(tb):
+                list.append('  File "%s", line %s, in %s' % tup)
+                line = linecache.getline(tup[0], tup[1]).rstrip()
+                if line:
+                    list.append(line)
+            htmlexc = '\n'.join(list)
+        finally:
+            del etype, value, tb
         return pyexc, htmlexc
 
     def handle_exception(self, ctx, req):
