@@ -60,14 +60,47 @@ class ResourceMixin:
         return self.__macros.get(name)
 
     def register_macro(self, name, macro):
+        existing = self.__macros.get(name)
+        if existing:
+            try:
+                existing_loc = existing.filename, existing.line_num
+                macro_loc = macro.filename, macro.line_num
+            except AttributeError:
+                pass
+            else:
+                if existing_loc != macro_loc:
+                    raise ApplicationError('macro %r already defined in %s:%s' %
+                                           (name, existing_loc[0], 
+                                            existing_loc[1]))
         self.__macros[name] = macro
 
     def get_lookup(self, name):
         return self.__lookups.get(name)
 
     def register_lookup(self, name, lookup):
+        existing = self.__lookups.get(name)
+        if existing:
+            try:
+                existing_loc = existing.filename, existing.line_num
+                lookup_loc = lookup.filename, lookup.line_num
+            except AttributeError:
+                pass
+            else:
+                if existing_loc != lookup_loc:
+                    raise ApplicationError('lookup %r already defined in %s:%s'%
+                                           (name, existing_loc[0], 
+                                            existing_loc[1]))
         self.__lookups[name] = lookup
 
+    def discard_file_resources(self, filename):
+        if filename is not None:
+            for name, macro in self.__macros.items():
+                if getattr(macro, 'filename', None) == filename:
+                    del self.__macros[name]
+            for name, lookup in self.__lookups.items():
+                if getattr(lookup, 'filename', None) == filename:
+                    del self.__lookups[name]
+
     def get_tagclass(self, name):
         return self.__tags.get(name)
 
@@ -155,6 +188,7 @@ class TemplateLoaderMixin:
             text = open(path).read()
         except IOError, e:
             raise TemplateLoadError("%s: %s" % (path, e.strerror))
+        self.discard_file_resources(path)
         return Template(self, path, text)
 
     def load_template_once(self, name):
@@ -188,6 +222,7 @@ class CachingTemplateLoaderMixin:
                 text = open(path).read()
             except IOError, e:
                 raise TemplateLoadError(e.strerror)
+            self.discard_file_resources(path)
             templ = Template(self, path, text)
             templ.__mtime__ = mtime
             self.__cache[path] = templ
@@ -258,9 +293,9 @@ class NameRecorderMixin:
         if have_zlib:
             text = zlib.compress(text)
         text = base64.encodestring(text)
-        self.write_content('<input type="hidden" name="__albform__" value="')
+        self.write_content('<div><input type="hidden" name="__albform__" value="')
         self.write_content(text)
-        self.write_content('" />\n')
+        self.write_content('" /></div>\n')
         self.__elem_names = {}
         return self.__need_multipart_enc
 
@@ -565,7 +600,7 @@ class SessionBase:
         self.clear_locals()
 
     def decode_session(self, text):
-        def imp_hook(name, globals, locals, fromlist):
+        def imp_hook(name, globals=None, locals=None, fromlist=None):
             if self.app.is_page_module(name):
                 return self.app.load_page_module(self, name)
             else:
@@ -633,9 +668,9 @@ class HiddenFieldSessionMixin(SessionBase):
     def form_close(self):
         if self.should_save_session():
             text = self.encode_session()
-            self.write_content('<input type="hidden" name="__albstate__" value="')
+            self.write_content('<div><input type="hidden" name="__albstate__" value="')
             self.write_content(text)
-            self.write_content('" />\n')
+            self.write_content('" /></div>\n')
 
 
 # ------------------------------------------------------------------
