@@ -33,7 +33,9 @@ class Request(cgiapp.Request):
     def get_header(self, name):
         env_name = 'HTTP_' + name.upper().replace('-', '_')
         return self.__fcgi.env.get(env_name)
-                                                     
+
+    def return_code(self):
+        self.__fcgi.Finish()
 
 def running():
     return fcgi.isFCGI()
