@@ -52,7 +52,7 @@ class SessionFileContextMixin(SessionCookieMixin, SessionBase):
             except:
                 self.app.del_session(sesid)
                 raise
-        self._set_sesid_cookie(self.__sesid, self.app.ses_age())
+        self._set_sesid_cookie(self.__sesid)
 
     def save_session(self):
         if self.should_save_session() and self.__sesid is not None:
@@ -65,7 +65,7 @@ class SessionFileContextMixin(SessionCookieMixin, SessionBase):
             self.app.del_session(self.__sesid)
             sesid = self.app.new_session()
             self.__sesid = sesid
-            self._set_sesid_cookie(self.__sesid, self.app.ses_age())
+            self._set_sesid_cookie(self.__sesid)
 
 
 class SessionFileAppMixin:
