@@ -7,7 +7,8 @@
 #
 import time
 
-from albatross.template import Content, EmptyTag, EnclosingTag
+from albatross.template import Content, EmptyTag, EnclosingTag, \
+                               template_version, template_features
 from albatross.common import *
 
 # Empty content object which tags can use as default value for
@@ -20,36 +21,15 @@ def escape(text):
     text = str(text)
     text = text.replace('&', '&amp;')
     text = text.replace('<', '&lt;')
-    text = text.replace('>', '&gt;',)
+    text = text.replace('>', '&gt;')
     text = text.replace('"', '&quot;')
     text = text.replace("'", '&#39;')
     return text
 
 
 # Implement the expr, nameattr, etc attribute behaviour
-class EvalAttribMixin:
+class InputNameValueMixin:
 
-    def assert_has_attribs(self, *attrs):
-        """ Check that the tag has at least one of the attributes """
-        for attr in attrs:
-            if self.has_attrib(attr):
-                return
-        if len(attrs) == 1:
-            self.raise_error('missing %s attribute' % repr(attrs[0]))
-        else:
-            attr_list = ', '.join([repr(attr) for attr in attrs[:-1]]) 
-            self.raise_error('missing %s or %s attribute' %\
-                             (attr_list, repr(attrs[-1])))
-
-    def eval_attrib(self, ctx, attr, kind = 'eval'):
-        code_attr = 'co_' + attr
-        try:
-            code_obj = getattr(self, code_attr)
-        except AttributeError:
-            code_obj = compile(self.get_attrib(attr), '<albatross>', kind)
-            setattr(self, code_attr, code_obj)
-        return ctx.eval_expr(code_obj)
-            
     def get_name(self, ctx):
         alias = self.get_attrib('alias')
         if alias:
@@ -79,7 +59,7 @@ class EvalAttribMixin:
 # the VALUE attribute.  When using Albatross application objects, the
 # corresponding browser request is automatically merged back into the
 # execution context.
-class Input(EmptyTag, EvalAttribMixin):
+class Input(EmptyTag, InputNameValueMixin):
 
     name = 'al-input'
 
@@ -96,7 +76,7 @@ class Input(EmptyTag, EvalAttribMixin):
             self.unbound_to_html = self.type_dict[self.__itype]
         except KeyError:
             self.raise_error('unrecognised <INPUT TYPE="%s">' % self.__itype)
-        self.assert_has_attribs(*self.naming_attrs)
+        self.assert_any_attrib(*self.naming_attrs)
 
     def write_attribs_except(self, ctx, *attribs):
         EmptyTag.write_attribs_except(self, ctx, *(self.all_attrs + attribs))
@@ -115,7 +95,7 @@ class Input(EmptyTag, EvalAttribMixin):
                     iter = ctx.get_value(iter_name)
                     node = iter.value()
                 return '%s,%s,%s' % (name, iter_name, node.albatross_alias())
-        return EvalAttribMixin.get_name(self, ctx)
+        return InputNameValueMixin.get_name(self, ctx)
 
     def get_name_value_checked(self, ctx, default = None):
         # Name
@@ -195,7 +175,7 @@ class Input(EmptyTag, EvalAttribMixin):
 
 # Allows the application need to modify the HREF attribute of the <A>
 # tag to include extra information from the execution context.
-class Href(EnclosingTag, EvalAttribMixin):
+class Href(EnclosingTag, InputNameValueMixin):
 
     name = 'al-a'
 
@@ -247,13 +227,13 @@ class Href(EnclosingTag, EvalAttribMixin):
 
 # Allows the application need to modify the SRC attribute of the <IMG>
 # tag to include extra information from the execution context.
-class Img(EmptyTag, EvalAttribMixin):
+class Img(EmptyTag, InputNameValueMixin):
 
     name = 'al-img'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
 
     def to_html(self, ctx):
         ctx.write_content('<img')
@@ -341,20 +321,20 @@ class Option(EnclosingTag):
 #  <al-option>12</al-option>
 # </al-select>
 # <al-select name="var" optionexpr="range(12)"/>
-class Select(EnclosingTag, EvalAttribMixin):
+class Select(EnclosingTag, InputNameValueMixin):
 
     name = 'al-select'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('name', 'alias', 'nameexpr')
+        self.assert_any_attrib('name', 'alias', 'nameexpr')
         self.options = []
 
     def to_html(self, ctx):
         name, value = self.get_name_and_value(ctx)
         ctx.write_content('<select')
-        self.write_attribs_except(ctx, 'optionexpr', 'alias', 'name',
-                                  'nameexpr', 'noescape')
+        self.write_attribs_except(ctx, 'expr', 'optionexpr', 'alias', 'name',
+                                  'nameexpr', 'noescape', 'list')
         ctx.write_content(' name="%s">' % name)
         if type(value) in (type([]), type(())):
             value = map(str, value)
@@ -402,13 +382,13 @@ class Select(EnclosingTag, EvalAttribMixin):
             EnclosingTag.append(self, tag)
 
 
-class TextArea(EnclosingTag, EvalAttribMixin):
+class TextArea(EnclosingTag, InputNameValueMixin):
 
     name = 'al-textarea'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('name', 'alias', 'nameexpr')
+        self.assert_any_attrib('name', 'alias', 'nameexpr')
 
     def to_html(self, ctx):
         name = self.get_name(ctx)
@@ -453,13 +433,13 @@ class Comment(EnclosingTag):
 
 # Include a template file. Can name the file directly, or can use an
 # expression to yield a filename.
-class Include(EmptyTag, EvalAttribMixin):
+class Include(EmptyTag, InputNameValueMixin):
 
     name = 'al-include'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('name', 'expr')
+        self.assert_any_attrib('name', 'expr')
 
     def to_html(self, ctx):
         if self.has_attrib('expr'):
@@ -482,13 +462,13 @@ class Include(EmptyTag, EvalAttribMixin):
 # <al-else>
 #  Value <= 5
 # </al-if>
-class Elif(EmptyTag, EvalAttribMixin):
+class Elif(EmptyTag, InputNameValueMixin):
 
     name = 'al-elif'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
 
     def is_true(self, ctx):
         return self.eval_attrib(ctx, 'expr')
@@ -499,13 +479,13 @@ class Else(EmptyTag):
     name = 'al-else'
 
 
-class If(EnclosingTag, EvalAttribMixin):
+class If(EnclosingTag, InputNameValueMixin):
 
     name = 'al-if'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
         self.elif_list = []
         self.else_content = not_specified
         self.current_content = self.content
@@ -534,13 +514,13 @@ class If(EnclosingTag, EvalAttribMixin):
 # Execute some Python code
 #
 # <al-exec expr="keys = os.environ.keys(); keys.sort()">
-class Exec(EmptyTag, EvalAttribMixin):
+class Exec(EmptyTag, InputNameValueMixin):
 
     name = 'al-exec'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
 
     def to_html(self, ctx):
         self.eval_attrib(ctx, 'expr', 'exec')
@@ -677,13 +657,13 @@ class ListIterator:
 #   Items are arranged to flow down columns by default.
 # FLOW:across/down
 #   Define the order in which multicolumn output is arranged.
-class For(EnclosingTag, EvalAttribMixin):
+class For(EnclosingTag, InputNameValueMixin):
 
     name = 'al-for'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('iter')
+        self.assert_any_attrib('iter')
         if self.has_attrib('flow'):
             if self.get_attrib('flow').lower() not in ('across', 'down'):
                 self.raise_error('"flow" must be "across" or "down"')
@@ -968,14 +948,14 @@ class EllipsisTreeIterator(LazyTreeIterator):
 
 
 # Format tree structured data.
-class Tree(EnclosingTag, EvalAttribMixin):
+class Tree(EnclosingTag, InputNameValueMixin):
 
     name = 'al-tree'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
-        self.assert_has_attribs('iter')
+        self.assert_any_attrib('expr')
+        self.assert_any_attrib('iter')
 
     def to_html(self, ctx):
         # Get iterator out of context - allows tree state to be
@@ -1055,13 +1035,13 @@ class Tree(EnclosingTag, EvalAttribMixin):
 # passed to time.strftime() to format the expression value.  The
 # LOOKUP value uses the expression value as an index into the named
 # lookup table and renders the value as the corresponding table entry.
-class Value(EmptyTag, EvalAttribMixin):
+class Value(EmptyTag, InputNameValueMixin):
 
     name = 'al-value'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
 
     def to_html(self, ctx):
         value = self.eval_attrib(ctx, 'expr')
@@ -1093,13 +1073,13 @@ class Value(EmptyTag, EvalAttribMixin):
 #  which the item EXPR attributes can be resolved to a
 #  constant.
 # </al-item>
-class Item(EnclosingTag, EvalAttribMixin):
+class Item(EnclosingTag, InputNameValueMixin):
 
     name = 'al-item'
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attribs('expr')
+        self.assert_any_attrib('expr')
 
 
 # Specify a lookup table for translating a value to arbitrary content
@@ -1115,8 +1095,9 @@ class Lookup(EnclosingTag):
 
     def __init__(self, ctx, filename, line_num, attribs):
         EnclosingTag.__init__(self, ctx, filename, line_num, attribs)
-        self.assert_has_attrib('name')
-        ctx.register_lookup(self.get_attrib('name'), self)
+        self.assert_any_attrib('name', 'expr')
+        if self.has_attrib('name'):
+            ctx.register_lookup(self.get_attrib('name'), self)
         self.item_list = []
 
     def append(self, tag):
@@ -1134,6 +1115,8 @@ class Lookup(EnclosingTag):
 
     def to_html(self, ctx):
         self._build_item_dict(ctx)
+        if self.has_attrib('expr'):
+            self.lookup_html(ctx, self.eval_attrib(ctx, 'expr'))
 
     def lookup_html(self, ctx, value):
         self._build_item_dict(ctx)
@@ -1207,7 +1190,29 @@ class Expand(EnclosingTag):
         macro.expand_html(ctx)
         ctx.pop_macro_args()
 
+class Require(EmptyTag):
+    
+    name = 'al-require'
+
+    def __init__(self, ctx, filename, line_num, attribs):
+        EmptyTag.__init__(self, ctx, filename, line_num, attribs)
+        self.assert_only_attrib('version', 'feature')
+        version = self.get_attrib('version')
+        if version:
+            try:
+                version = int(version)
+            except ValueError:
+                self.raise_error('version attribute must be an integer')
+            if version > template_version:
+                self.raise_error('template requires features from templating version %d, this is version %d' % (version, template_version))
+        feature = self.get_attrib('feature')
+        if feature:
+            for feature in feature.split(','):
+                feature = feature.strip()
+                if feature not in template_features:
+                    self.raise_error('template requires feature %r - not available' % feature)
+
 
 tags = (Input, Href, Img, Form, Option, Select, TextArea,
         Flush, Comment, Include, Elif, Else, If, Exec, For, Tree,
-        Value, Item, Lookup, Usearg, Macro, Setarg, Expand)
+        Value, Item, Lookup, Usearg, Macro, Setarg, Expand, Require)
