@@ -6,7 +6,7 @@
 #
 # Dummy Request objects
 #
-# $Id: request.py,v 1.5 2003/01/27 10:23:27 djc Exp $
+# $Id: request.py,v 1.7 2004/10/19 12:57:36 andrewm Exp $
 #
 
 import string, re
@@ -18,6 +18,9 @@ class DummyCookieRequest:
     def __init__(self):
         self.cookie = None
 
+    def get_method(self):
+        return 'GET'
+
     def write_header(self, hdr, value):
         if hdr == 'Set-Cookie':
             self.cookie = value
@@ -31,7 +34,7 @@ class DummyCookieRequest:
 
 class DummyFieldRequest:
     def __init__(self):
-        self.fieldre = re.compile(r'<input type="hidden" name="(__al[a-zA-Z0-9]*__)" value="([^"]*)">', re.I|re.M|re.S)
+        self.fieldre = re.compile(r'<input type="hidden" name="(__al[a-zA-Z0-9]*__)" value="([^"]*)" />', re.I|re.M|re.S)
         self.fields = {}
         self.__content_parts = []
 
